package djmil.cordacheckers.api;

import java.util.List;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import djmil.cordacheckers.cordaclient.CordaClient;
import djmil.cordacheckers.cordaclient.dao.GameView;
import djmil.cordacheckers.user.HoldingIdentityResolver;
import djmil.cordacheckers.user.User;


@RestController
@RequestMapping("api/game")
public class GameController {

    @Autowired
    CordaClient cordaClient;

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    @GetMapping
    public ResponseEntity<List<GameView>> list(
        @AuthenticationPrincipal User player
    ) {
        final List<GameView> gsList = cordaClient.gameStateList(player.getHoldingIdentity());

        return ResponseEntity.ok(gsList);
    }

    @PutMapping("/{uuid}/surrender") 
    public ResponseEntity<GameView> surrender(
        @AuthenticationPrincipal User issuer,
        @PathVariable UUID uuid
    ) {
        final GameView finishedGame = cordaClient.gameBoardSurrender(
            issuer.getHoldingIdentity(), 
            uuid
        );

        return ResponseEntity
                .ok(finishedGame);
    }

    @PutMapping("/{uuid}/drawreq") 
    public ResponseEntity<GameView> drawRequest(
        @AuthenticationPrincipal User issuer,
        @PathVariable UUID uuid
    ) {
        final GameView drawReqGame = cordaClient.gameDrawRequest(
            issuer.getHoldingIdentity(), 
            uuid
        );

        return ResponseEntity
                .ok(drawReqGame);
    }

    @PutMapping("/{uuid}/drawacc") 
    public ResponseEntity<GameView> drawAccept(
        @AuthenticationPrincipal User issuer,
        @PathVariable UUID uuid
    ) {
        final GameView drawAccGame = cordaClient.gameDrawAccept(
            issuer.getHoldingIdentity(), 
            uuid
        );

        return ResponseEntity
                .ok(drawAccGame);
    }

    @PutMapping("/{uuid}/drawrej") 
    public ResponseEntity<GameView> drawReject(
        @AuthenticationPrincipal User issuer,
        @PathVariable UUID uuid
    ) {
        final GameView drawRejGame = cordaClient.gameDrawReject(
            issuer.getHoldingIdentity(), 
            uuid
        );

        return ResponseEntity
                .ok(drawRejGame);
    }
}
