import { usePolling, doPushing } from '../hook/api';
import { gamesInitialState } from '../reducer/games';

export default function useGamesApi(gamesReducer, config) {
    const [games, dispatchGames] = gamesReducer;

    const usePollingGamesList = () => {
        const onSuccess = (gamesList) => {
            dispatchGames({ type: 'next', gamesList });
        }

        const isPollingGamesList = usePolling('/api/game', onSuccess, config.intervalMode(30));
        if (games.isPollingGamesList !== isPollingGamesList) {
            dispatchGames({ type: 'next', isPollingGamesList });
        }

        return games;
    }

    return {
        pollGamesList: usePollingGamesList,

        pushNewGame: ({ opponentName, opponentColor, board, message }) => ifNot(games.isPushingNewGame) &&
            doPushing('/api/gameproposal', 'POST', { opponentName, opponentColor, board, message }, {
                onPushing: (isPushingNewGame) => dispatchGames({ type: 'next', isPushingNewGame }),
                onSuccess: (game) => dispatchGames({ type: 'next', gamesList: [game, ...games.gamesList], newGame: gamesInitialState.newGame })
            }),

        pushGameProposalCancel: ({ uuid }) => ifNot(games.isPushingGameProposalCancel) &&
            doPushing(`/api/gameproposal/${uuid}/cancel`, 'PUT', null, {
                onPushing: (isPushingGameProposalCancel) => dispatchGames({ type: 'next', isPushingGameProposalCancel }),
                onSuccess: (canceledGame) => dispatchGames({ type: 'next', gamesList: games.nextGameList(canceledGame), proposal: gamesInitialState.proposal })
            }),

        pushGameProposalReject: ({ uuid }) => ifNot(games.isPushingGameProposalReject) &&
            doPushing(`/api/gameproposal/${uuid}/reject`, 'PUT', null, {
                onPushing: (isPushingGameProposalReject) => dispatchGames({ type: 'next', isPushingGameProposalReject }),
                onSuccess: (rejectedGame) => dispatchGames({ type: 'next', gamesList: games.nextGameList(rejectedGame), proposal: gamesInitialState.proposal })
            }),

        pushGameProposalAccept: ({ uuid }) => ifNot(games.isPushingGameProposalAccept) &&
            doPushing(`/api/gameproposal/${uuid}/accept`, 'PUT', null, {
                onPushing: (isPushingGameProposalAccept) => dispatchGames({ type: 'next', isPushingGameProposalAccept }),
                onSuccess: (acceptedGame) => dispatchGames({ type: 'next', gamesList: games.nextGameList(acceptedGame), proposal: gamesInitialState.proposal })
            }),

        pushGameSurrender: ({ uuid }) => ifNot(games.isPushingGameSurrender) &&
            doPushing(`/api/game/${uuid}/surrender`, 'PUT', null, {
                onPushing: (isPushingGameSurrender) => dispatchGames({ type: 'next', isPushingGameSurrender }),
                onSuccess: (finishedGame) => dispatchGames({ type: 'next', gamesList: games.nextGameList(finishedGame), proposal: gamesInitialState.active })
            }),

        pushGameDrawRequest: ({ uuid }) => ifNot(games.isPushingGameDrawRequest) &&
            doPushing(`/api/game/${uuid}/drawreq`, 'PUT', null, {
                onPushing: (isPushingGameDrawRequest) => dispatchGames({ type: 'next', isPushingGameDrawRequest }),
                onSuccess: (drawReqGame) => dispatchGames({ type: 'next', gamesList: games.nextGameList(drawReqGame), proposal: gamesInitialState.active })
            }),
        
        pushGameDrawAccept: ({ uuid }) => ifNot(games.isPushingGameDrawAccept) &&
            doPushing(`/api/game/${uuid}/drawacc`, 'PUT', null, {
                onPushing: (isPushingGameDrawAccept) => dispatchGames({ type: 'next', isPushingGameDrawAccept }),
                onSuccess: (drawAccGame) => dispatchGames({ type: 'next', gamesList: games.nextGameList(drawAccGame), proposal: gamesInitialState.active })
            }),

        pushGameDrawReject: ({ uuid }) => ifNot(games.isPushingGameDrawReject) &&
            doPushing(`/api/game/${uuid}/drawrej`, 'PUT', null, {
                onPushing: (isPushingGameDrawReject) => dispatchGames({ type: 'next', isPushingGameDrawReject }),
                onSuccess: (drawRejGame) => dispatchGames({ type: 'next', gamesList: games.nextGameList(drawRejGame), proposal: gamesInitialState.active })
            }),
    }
}

function ifNot(expression) {
    return !expression;
}