import React, { useContext } from 'react';
import { GamesContext } from '../../context/games';
import Wobler from '../../components/Wobler';

/*
 * NewGame actoins
 */
export function Create({ onClick }) {
    const games = useContext(GamesContext);

    const hasOpponent = games.newGame.opponentName && games.newGame.opponentColor;

    const prepareNewGameRequest = () => {
        if (!hasOpponent)
            return alert("You have to select an opponent");

        const reqParams = {
            opponentName: games.newGame.opponentName,
            opponentColor: games.newGame.opponentColor,
            board: null, // default board configuration
            message: games.newGame.message
        }

        onClick(reqParams);
    }

    return (
        <button className={'Create' + (hasOpponent ? ' ready' : '')}
            onClick={prepareNewGameRequest}
        >
            <Wobler text="Create" dance={games.isPushingNewGame} />
        </button>
    )
}

/*
 * GameProposal actions
 */

export function Accept({ onClick }) {
    const games = useContext(GamesContext);

    const selectedGame = games.findGame({ uuid: games.proposal.selectedUUID });
    const isReady = selectedGame?.status === 'GAME_PROPOSAL_WAIT_FOR_YOU' ? true : '';

    if (selectedGame?.status !== 'GAME_PROPOSAL_WAIT_FOR_OPPONENT')
        return (
            <button className={'Accept' + (isReady && ' ready')}
                onClick={() => isReady ? onClick(selectedGame.uuid) : alert('You have to select some GameProposal')}
            >
                <Wobler text="Accept" dance={games.isPushingGameProposalAccept} />
            </button>
        )
}

export function Reject({ onClick }) {
    const games = useContext(GamesContext);

    const selectedGame = games.findGame({ uuid: games.proposal.selectedUUID });
    const isReady = selectedGame?.status === 'GAME_PROPOSAL_WAIT_FOR_YOU' ? true : '';

    if (selectedGame?.status !== 'GAME_PROPOSAL_WAIT_FOR_OPPONENT')
        return (
            <button className={'Reject' + (isReady && ' ready')}
                onClick={() => isReady ? onClick(selectedGame.uuid) : alert('You have to select some GameProposal')}
            >
                <Wobler text="Reject" dance={games.isPushingGameProposalReject} />
            </button>
        )
}

export function Cancel({ onClick }) {
    const games = useContext(GamesContext);

    const selectedGame = games.findGame({ uuid: games.proposal.selectedUUID });
    const isReady = selectedGame?.status === 'GAME_PROPOSAL_WAIT_FOR_OPPONENT' ? true : '';

    if (selectedGame?.status === 'GAME_PROPOSAL_WAIT_FOR_OPPONENT')
        return (
            <button className={'Cancel' + (isReady && ' ready')}
                onClick={() => isReady ? onClick(selectedGame.uuid) : alert('You have to select pending GameProposal')}
            >
                <Wobler text="Cancel" dance={games.isPushingGameProposalCancel} />
            </button>
        )
}

/*
 * Game actions
 */

export function Surrender({ onClick }) {
    const games = useContext(GamesContext);

    const selectedGame = games.findGame({ uuid: games.active.selectedUUID });
    const gameStatus = selectedGame?.status;
    const isReady = (gameStatus === 'GAME_BOARD_WAIT_FOR_OPPONENT' || gameStatus === 'GAME_BOARD_WAIT_FOR_YOU') ? true : '';

    if (gameStatus === 'DRAW_REQUEST_WAIT_FOR_YOU' || gameStatus === 'DRAW_REQUEST_WAIT_FOR_OPPONENT')
        return; // You shall not surrender if there is an active tie negotiations

    return (
        <button className={'Surrender' + (isReady && ' ready')}
            onClick={() => isReady ? onClick(selectedGame.uuid) : alert('You have to select a game')}
        >
            <Wobler text="Surrender" dance={games.isPushingGameSurrender} />
        </button>
    )
}

/*
 * Game actions: Draw
 */

export function DrawRequest({ onClick }) {
    const games = useContext(GamesContext);

    const selectedGame = games.findGame({ uuid: games.active.selectedUUID });
    const gameStatus = selectedGame?.status;
    const isReady = gameStatus === 'GAME_BOARD_WAIT_FOR_YOU' ? true : '';

    const checkStatus = () => {
        if (!selectedGame)
            return alert('You have to select a game');

        if (gameStatus === 'DRAW_REQUEST_WAIT_FOR_OPPONENT')
            return alert('A draw was alredy offered to the opponent');

        if (!isReady)
            return alert('You can ask for a draw only during your turn');

        onClick(selectedGame.uuid);
    }

    if (gameStatus === 'DRAW_REQUEST_WAIT_FOR_YOU')
        return; // You can not send counter draw request

    return (
        <button className={'Draw' + (isReady && ' ready')} onClick={() => checkStatus()} >
            <Wobler text="Draw?" dance={games.isPushingGameDrawRequest} />
        </button >
    )
}

export function DrawAccept({ onClick }) {
    const games = useContext(GamesContext);

    const selectedGame = games.findGame({ uuid: games.active.selectedUUID });
    const gameStatus = selectedGame?.status;

    if (gameStatus === 'DRAW_REQUEST_WAIT_FOR_YOU')
        return (
            <button className='Draw accept' onClick={() => onClick(selectedGame.uuid)} >
                <Wobler text="Draw accept" dance={games.isPushingGameDrawAccept} />
            </button>
        )
}

export function DrawReject({ onClick }) {
    const games = useContext(GamesContext);

    const selectedGame = games.findGame({ uuid: games.active.selectedUUID });

    if (selectedGame?.status === 'DRAW_REQUEST_WAIT_FOR_YOU')
        return (
            <button className='Draw reject' onClick={() => onClick(selectedGame.uuid)} >
                <Wobler text="Reject" dance={games.isPushingGameDrawReject} />
            </button>
        )
}

/*
 * GameArchive actions
 */

export function Backward() {

    return <button className='Backward' disabled>Backward</button>
}

export function Forward() {

    return <button className='Forward' disabled>Forward</button>
}